#!/usr/bin/perl -w
#
#  Copyright 2000, G. Wade Johnson
#  Released under the same license as Perl.
#
#  Version: 0.7.1
#
#  Calculate metrics from a WSDL file.
#

use FindBin;
use lib "$FindBin::Bin/../lib";
use WSDL;
use Data::Dumper;
use Getopt::Std;
use strict;

my @Summary =
 (
  [ "total navigation elements:  ", '_navs',             ],
  [ " group elements:            ", 'group',             ],
  [ " page elements:             ", 'page',              ],
  [ " pageref elements:          ", 'pageref',           ],
  [ " fpage elements:            ", 'fpage',             ],
  [ " subpage elements:          ", 'subpage',           ],
  [ " website elements:          ", 'website',           ],
  [ "max elements per navigation:", '_max_nav',          ],
  [ "max elements per group:     ", '_max_grp',   "\n\n" ],
  [ "layout elements:            ", 'layout',            ],
  [ "header elements:            ", 'header',            ],
  [ "footer elements:            ", 'footer',            ],
  [ "code elements:              ", 'code',              ],
  [ "text elements:              ", 'text',              ],
  [ "stylesheet elements:        ", 'stylesheet', "\n\n" ],
  [ "max depth of nesting:       ", '_nesting',   "\n\n" ],
 );

my @NavItem = qw/group page pageref subpage fpage file website/;

my $nest = 0;

#--------------------------------------------
#  Begin main code

my %CmdLine = ();
my %Counts  = ();
extract_defines();

die "No WSDL file specified.\n" unless @ARGV;

my $file = shift;

my $wsdl = new WSDL( $file, %CmdLine );

count_elements( $wsdl->website );

print "WSDL Summary:\n------------\n";

foreach my $sum (@Summary)
 {
  print " ", $sum->[0], " ", ($Counts{$sum->[1]} || 0), ($sum->[2]||"\n");
 }

#---------------------------------------------
#  Subroutines

sub  extract_defines
 {
  while(@ARGV)
   {
    last unless $ARGV[0] =~ /(\w+)=(.*)/;
    
    $CmdLine{$1} = $2;
    shift @ARGV;
   }
 }


sub count_elements
 {
  my $elem = shift;

  $Counts{_nesting} = 0;
  $Counts{_max_nav} = 0;
  $Counts{_max_grp} = 0;

  _count_elements( $elem );

  $Counts{_navs}   = 0;

  foreach(@NavItem)
   {
    $Counts{_navs} += $Counts{$_} if exists $Counts{$_};
   }
 }




sub _count_elements
 {
  my $elem = shift;

  $nest++;
  $Counts{_nesting} = $nest if $nest > $Counts{_nesting};

  foreach my $e ($elem->ElementContent())
   {
    $Counts{$e->Type()}++;

    _count_elements( $e ) unless $e->IsEmpty;
   }

  if('navigation' eq $elem->Type())
   {
    my $num = $elem->ElementContent();
    $Counts{_max_nav} = $num if $Counts{_max_nav} < $num;
   }

  if('group' eq $elem->Type())
   {
    my $num = $elem->ElementContent();
    $Counts{_max_grp} = $num if $Counts{_max_grp} < $num;
   }

  $nest--;
 }
